package work.xuye.scheduler.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.stream.function.StreamBridge;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import work.xuye.common.constant.BindingConstants;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.constant.StageConstants;
import work.xuye.common.db.service.TaskManager;
import work.xuye.common.db.service.TaskService;
import work.xuye.common.dto.HttpRequestParams;
import work.xuye.common.dto.TaskVO;
import work.xuye.common.enums.ProcessMode;
import work.xuye.common.utils.ID;

import java.util.ArrayList;

/**
 * @author xuye
 * @since 2023/3/12 22:57
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class IssueService {

    private final StreamBridge streamBridge;

    private final TaskManager taskManager;

    private final TaskService taskService;

    public TaskVO issueTask(Integer id) {
        TaskVO taskVO = taskManager.getTaskInfoByTaskId(id);
        HttpRequestParams requestParams = taskVO.getTask().getRequestParams();
        Message<HttpRequestParams> message = MessageBuilder
                .withPayload(requestParams)
                .setHeader(MessageConstants.CURRENT_STAGE, StageConstants.SCHEDULER)
                .setHeader(MessageConstants.TASK_TRACE_ID, ID.generate())
                .setHeader(MessageConstants.TASK_ID, taskVO.getTask().getId())
                .setHeader(MessageConstants.TASK_NAME, taskVO.getTask().getName())
                .build();
        streamBridge.send(BindingConstants.TASK_OUT, message);
        ProcessMode processMode = taskVO.getTask().getProcessMode();
        String emoji = processMode.equals(ProcessMode.DEBUG) ? "\uD83E\uDD16" : "";
        log.info("\uD83D\uDCE4 {} [{}-{}], " + MessageConstants.TASK_TRACE_ID + ": [{}]" + ", processMode: [{}]",
                emoji,
                taskVO.getTask().getId(),
                taskVO.getTask().getName(),
                message.getHeaders().get(MessageConstants.TASK_TRACE_ID),
                processMode);
        return taskVO;
    }

    @Scheduled(cron = "0 0/1 * * * ?")
    public ArrayList<TaskVO> issueAllTask() {

        ArrayList<TaskVO> result = new ArrayList<>();

        taskService.list().forEach(task -> {
            result.add(this.issueTask(task.getId()));
        });
        return result;
    }
}
