package work.xuye.sink.service;

import com.alibaba.druid.pool.DruidDataSource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;
import work.xuye.common.db.entity.Datasource;
import work.xuye.common.db.service.DatasourceService;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author xuye
 * @since 2023/3/4 21:54
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class JdbcService {

    private final DatasourceService datasourceService;

    private Map<String, DataSource> datasourceMap = new HashMap<>();
    private Map<String, JdbcTemplate> jdbcTemplateMap = new HashMap<>();


    private void refresh() {
        List<Datasource> datasourceList = datasourceService.list();
        for (Datasource datasource : datasourceList) {
            if (!datasourceMap.containsKey(datasource.getName())) {
                DruidDataSource dataSource = new DruidDataSource();
                dataSource.setUrl(datasource.getUrl());
                dataSource.setUsername(datasource.getUsername());
                dataSource.setPassword(datasource.getPassword());
                dataSource.setDriverClassName(datasource.getDriverClassName());
                datasourceMap.put(datasource.getName(), dataSource);
                jdbcTemplateMap.put(datasource.getName(), new JdbcTemplate(dataSource));
            }
            log.info("datasource add success:{}", datasource.getName());
        }
    }


    public boolean isExist(String datasourceName, String sql) {
        JdbcTemplate jdbcTemplate = this.getJdbcTemplate(datasourceName);
        Integer integer = jdbcTemplate.queryForObject(sql, Integer.class);
        if (ObjectUtils.isEmpty(integer)) {
            return false;
        }
        return integer > 0;
    }


    public void update(String datasourceName, String sql) {
        JdbcTemplate jdbcTemplate = this.getJdbcTemplate(datasourceName);
        int update = jdbcTemplate.update(sql);
        if (update == 1) {
            log.debug("sql执行成功:{}", sql);
        } else {
            log.error("sql执行失败:{}", sql);
        }

    }


    public Map<String, Object> getOne(String datasourceName, String sql) {
        JdbcTemplate jdbcTemplate = this.getJdbcTemplate(datasourceName);
        return jdbcTemplate.queryForMap(sql);
    }


    public JdbcTemplate getJdbcTemplate(String datasourceName) {
        if (!jdbcTemplateMap.containsKey(datasourceName)) {
            this.refresh();
        }
        JdbcTemplate jdbcTemplate = jdbcTemplateMap.get(datasourceName);
        Assert.notNull(jdbcTemplate, "数据源 " + datasourceName + " 不存在");
        return jdbcTemplate;
    }

    public List<String> getForList(String datasourceName, String sql) {
        JdbcTemplate jdbcTemplate1 = this.getJdbcTemplate(datasourceName);
        return jdbcTemplate1.queryForList(sql, String.class);

    }
}
